package people;


public class TestPeopleDatabaseDynamic extends AbstractTest {

	public static void main(String[] args) throws InstantiationException, IllegalAccessException {
		new TestPeopleDatabaseDynamic();
	}

	protected void runTests() {
		this.testExpected = 45;
		try {
			testSuperTypes();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testEmptyDatabase();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testAddAndGetDatabase();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testGetCount();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testGetAverageSalary();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testGetStudentAverageAge();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testGetOldestPerson();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try { 
			testGetRichestEmployee();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
	}

	public void testSuperTypes() {
		PeopleDatabaseDynamicImpl p = new PeopleDatabaseDynamicImpl();
		assertInstanceOf(p, people.PeopleDatabase.class);
	}

	public void testEmptyDatabase() {
		PeopleDatabaseDynamicImpl p = new PeopleDatabaseDynamicImpl();

		assertEquals(0, p.getPersonCount());
		assertEquals(0, p.getEmployeeCount());
		assertEquals(0, p.getStudentCount());
		assertEquals(0., p.getAverageSalary());
		assertEquals(0., p.getStudentAverageAge());
		assertNull(p.getOldestPerson());
		assertNull(p.getRichestEmployee());
		assertNull(p.get("ANYID"));
	}

	public void testAddAndGetDatabase() {
		PeopleDatabaseDynamicImpl p = new PeopleDatabaseDynamicImpl();
		Student s = new StudentImpl("ESIAL2012", "Anakin", 19, Student.YEAR_1ST);
		Employee e = new EmployeeImpl("N2501UM3N", "Motoko Kusanagi", 21, 2501.);
		p.add(s);
		p.add(e);
		assertEquals(s, p.get("ESIAL2012"));
		assertEquals(e, p.get("N2501UM3N"));
		assertNull(p.get("unkown"));
	}

	public void testGetCount() {
		PeopleDatabaseDynamicImpl p = new PeopleDatabaseDynamicImpl();
		Student s = new StudentImpl("ESIAL2012", "Anakin", 19, Student.YEAR_1ST);
		Employee e = new EmployeeImpl("N2501UM3N", "Motoko Kusanagi", 21, 2501.);
		assertEquals(0, p.getPersonCount());
		assertEquals(0, p.getEmployeeCount());
		assertEquals(0, p.getStudentCount());
		p.add(s);
		assertEquals(1, p.getPersonCount());
		assertEquals(0, p.getEmployeeCount());
		assertEquals(1, p.getStudentCount());
		p.add(e);
		assertEquals(2, p.getPersonCount());
		assertEquals(1, p.getEmployeeCount());
		assertEquals(1, p.getStudentCount());
	}

	public void testGetAverageSalary() {
		PeopleDatabaseDynamicImpl p = new PeopleDatabaseDynamicImpl();

		assertTrue(!(new Double(p.getAverageSalary())).isNaN(), "did you try to divide by 0?");
		assertEquals(0., p.getAverageSalary());
		Employee e = new EmployeeImpl("N2501UM3N", "PHD", 21, 1200.);
		p.add(e);
		assertEquals(1200., p.getAverageSalary());
		Student s = new StudentImpl("JEDI2012", "Anakin", 19, Student.YEAR_1ST);
		p.add(s);
		assertEquals(1200., p.getAverageSalary());
		e = new EmployeeImpl("N2503UM3N", "MCF", 21, 1900.);
		p.add(e);
		assertEquals((1200. + 1900.) / 2, p.getAverageSalary());
		e = new EmployeeImpl("N2504UM3N", "PR", 21, 2500.);
		p.add(e);
		assertEquals((1200. + 1900. + 2500.) / 3, p.getAverageSalary());
	}

	public void testGetStudentAverageAge() {
		PeopleDatabaseDynamicImpl p = new PeopleDatabaseDynamicImpl();

		assertTrue(!(new Double(p.getStudentAverageAge())).isNaN(), "did you try to divide by 0?");
		assertEquals(0., p.getStudentAverageAge());
		Student s = new StudentImpl("JEDI2012", "Anakin", 19, Student.YEAR_1ST);
		p.add(s);
		assertEquals(19., p.getStudentAverageAge());
		Employee e = new EmployeeImpl("N2501UM3N", "PHD", 21, 1200.);
		p.add(e);
		assertEquals(19., p.getStudentAverageAge());
		s = new StudentImpl("REBEL2000", "Luke", 25, Student.YEAR_3RD);
		p.add(s);
		assertEquals((19. + 25.) / 2, p.getStudentAverageAge());
		s = new StudentImpl("REBEL2001", "Leia", 25, Student.YEAR_3RD);
		p.add(s);
		assertEquals((19. + 25. + 25.) / 3, p.getStudentAverageAge());
	}

	public void testGetOldestPerson() {
		PeopleDatabaseDynamicImpl p = new PeopleDatabaseDynamicImpl();

		assertNull(p.getOldestPerson());
		Person pe1 = new StudentImpl("JEDI2012", "Anakin", 19, Student.YEAR_1ST);
		p.add(pe1);
		assertEquals(pe1, p.getOldestPerson());
		Person pe2 = new EmployeeImpl("N2501UM3N", "PHD", 21, 1200.);
		p.add(pe2);
		assertEquals(pe2, p.getOldestPerson());
		Person pe3 = new StudentImpl("REBELL1977", "Yoda", 85, Student.YEAR_3RD);
		p.add(pe3);
		assertEquals(pe3, p.getOldestPerson());
		Person pe4 = new StudentImpl("JEDI2002", "Luke", 24, Student.YEAR_3RD);
		p.add(pe4);
		assertEquals(pe3, p.getOldestPerson());
		Person pe5 = new StudentImpl("EMPIRE1978", "Emperor", 85, Student.YEAR_3RD);
		p.add(pe5);
		assertEquals(pe3, p.getOldestPerson());
	}

	public void testGetRichestEmployee() {
		PeopleDatabaseDynamicImpl p = new PeopleDatabaseDynamicImpl();

		assertNull(p.getRichestEmployee());
		Person pe1 = new EmployeeImpl("N2501UM3N", "PHD", 21, 1200.);
		p.add(pe1);
		assertEquals(pe1, p.getRichestEmployee());
		Person pe2 = new StudentImpl("JEDI2012", "Anakin", 19, Student.YEAR_1ST);
		p.add(pe2);
		assertEquals(pe1, p.getRichestEmployee());
		Person pe3 = new EmployeeImpl("REBELL1977", "Yoda", 85, 1700.);
		p.add(pe3);
		assertEquals(pe3, p.getRichestEmployee());
		Person pe4 = new EmployeeImpl("JEDI2002", "Luke", 24, 600.);
		p.add(pe4);
		assertEquals(pe3, p.getRichestEmployee());
		Person pe5 = new EmployeeImpl("EMPIRE1978", "Emperor", 85, 1700.);
		p.add(pe5);
		assertEquals(pe5, p.getRichestEmployee());
	}

}
