package people;

public class TestPerson extends AbstractTest {

	public static void main(String[] args) throws InstantiationException, IllegalAccessException {
		new TestPerson();
	}

	protected void runTests() {
		this.testExpected = 20;
		try {
			testSuperTypes();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testGetName();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testGetAndSetAge();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testGetUniqueId();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testToString();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testCopyConstructor();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
		try {
			testEquals();
		} catch (Exception e) { e.printStackTrace(); /*this.testFailed++;*/}
	}

	class MyPerson extends people.PersonImpl {
		public MyPerson(String name, int age) {
			super(name, age);
		}

		public MyPerson(MyPerson other) {
			super(other);
		}

		public String getUniqueId() {
			return "Secret";
		}
	}

	public void testSuperTypes() {
		MyPerson p = new MyPerson("Elwood", 21);
		assertInstanceOf(p, people.PersonImpl.class);
		assertInstanceOf(p, people.Person.class);
	}

	public void testGetName() {
		MyPerson p = new MyPerson("Elwood", 21);

		String expectedS = "Elwood";
		String valueS = p.getName();
		assertEquals(expectedS, valueS);

		p = new MyPerson("Jack", 21);

		expectedS = "Jack";
		valueS = p.getName();
		assertEquals(expectedS, valueS);
	}

	public void testGetAndSetAge() {
		MyPerson p = new MyPerson("Elwood", 65);

		int expected = 65;
		int value = p.getAge();
		assertEquals(expected, value);

		p.setAge(23);
		expected = 23;
		value = p.getAge();
		assertEquals(expected, value);
	}

	public void testGetUniqueId() {
		MyPerson p = new MyPerson("Elwood", 65);

		assertEquals("Secret", p.getUniqueId());
	}

	public void testToString() {
		MyPerson p = new MyPerson("Elwood", 65);

		String expectedS = "[Secret] Elwood (age: 65)";
		String valueS = p.toString();
		assertEquals(expectedS, valueS);
	}

	public void testCopyConstructor() {
		MyPerson p = new MyPerson("Elwood", 65);
		MyPerson copy = new MyPerson(p);

		assertTrue(p != copy, "references should not be equal");
		assertEquals(p.getAge(), copy.getAge());
		assertEquals(p.getName(), copy.getName());
		assertEquals(p.getUniqueId(), copy.getUniqueId());
	}

	public void testEquals() {
		MyPerson p = new MyPerson("Elwood", 65);
		MyPerson p2 = new MyPerson("Elwood", 65);
		MyPerson pSameName = new MyPerson("Elwood", 55);
		MyPerson pSameAge = new MyPerson("Elwod", 65);
		MyPerson pNullName = new MyPerson(null, 66);
		MyPerson otherNullName = new MyPerson(null, 66);

		assertTrue(! p.equals(new Object()), "objects should not be equal");		
		
		assertTrue(p.equals(p), "object should be equal to itself");
		assertTrue(p.equals(p2), "objects should be equal");
		assertTrue(!p.equals(pSameName), "objects should not be equal");
		assertTrue(!p.equals(pSameAge), "objects should not be equal");
		assertTrue(!p.equals(null), "object should not be equal to null reference");
		assertTrue(!p.equals(pNullName), "objects should not be equal");
		assertTrue(otherNullName.equals(pNullName), "objects should be equal");
		
		// we might test getUniqueId() in equals() method of class PersonImpl
	}

}
